@@ -56,7 +56,7 @@ class SqliteDocumentPersistence(object):
     The Sqlite persistence engine to support large scale documents (part of the V4 Kodexa Document Architecture)
     """
 
-    def __init__(self, document: Document, filename: str = None, delete_on_close=False):
+    def __init__(self, document: Document, filename: str = None, delete_on_close=False, inmemory=False):
         self.document = document
 
         self.node_types = {}
@@ -84,12 +84,44 @@ class SqliteDocumentPersistence(object):
 
         self.current_filename = filename
 
-        self.connection = sqlite3.connect(filename)
+        if inmemory:
+            self.inmemory=True
+            self.connection = self.create_in_memory_database(filename)
+        else:
+            self.inmemory=False
+            self.connection = sqlite3.connect(filename)
+
         self.cursor = self.connection.cursor()
         self.cursor.execute("PRAGMA journal_mode=OFF")
         self.cursor.execute("pragma temp_store = memory")
         self.cursor.execute("pragma mmap_size = 30000000000")
 
+    def create_in_memory_database(self, disk_db_path: str):
+        # Connect to the in-memory database
+        mem_conn = sqlite3.connect(':memory:')
+        mem_cursor = mem_conn.cursor()
+
+        # Connect to the database on disk
+        disk_conn = sqlite3.connect(disk_db_path)
+        disk_cursor = disk_conn.cursor()
+
+        # Load the contents of the disk database into memory
+        disk_cursor.execute("SELECT name FROM sqlite_master WHERE type='table';")
+        tables = disk_cursor.fetchall()
+        for table in tables:
+            table_name = table[0]
+            disk_cursor.execute(f"SELECT * FROM {table_name}")
+            rows = disk_cursor.fetchall()
+            mem_cursor.execute(f"CREATE TABLE {table_name} AS SELECT * FROM {table_name}", [])
+            for row in rows:
+                mem_cursor.execute(f"INSERT INTO {table_name} VALUES ({', '.join('?' * len(row))})", row)
+
+        # Commit changes and close disk connection
+        mem_conn.commit()
+        disk_conn.close()
+
+        return mem_conn
+
     def get_all_tags(self):
         """
         Retrieves all tags from the document.
@@ -99,7 +131,7 @@ class SqliteDocumentPersistence(object):
         """
         features = []
         for feature in self.cursor.execute(
-            "select name from f_type where name like 'tag:%'"
+                "select name from f_type where name like 'tag:%'"
         ).fetchall():
             features.append(feature[0].split(":")[1])
 
@@ -500,7 +532,7 @@ class SqliteDocumentPersistence(object):
         """
         document_metadata = {
             "version": Document.CURRENT_VERSION,
-            "metadata": self.document.metadata.to_dict(),
+            "metadata": self.document.metadata,
             "source": self.__clean_none_values(
                 dataclasses.asdict(self.document.source)
             ),
@@ -698,8 +730,8 @@ class SqliteDocumentPersistence(object):
         # We need to get the child nodes
         children = []
         for child_node in self.cursor.execute(
-            "select id, pid, nt, idx from cn where pid = ? order by idx",
-            [content_node.uuid],
+                "select id, pid, nt, idx from cn where pid = ? order by idx",
+                [content_node.uuid],
         ).fetchall():
             children.append(self.__build_node(child_node))
         return children
@@ -718,8 +750,8 @@ class SqliteDocumentPersistence(object):
         # We need to get the child nodes
         children = []
         for child_node in self.cursor.execute(
-            "select id, pid, nt, idx from cn where pid = ? order by idx",
-            [content_node.uuid],
+                "select id, pid, nt, idx from cn where pid = ? order by idx",
+                [content_node.uuid],
         ).fetchall():
             children.append(child_node[0])
         return children
@@ -787,14 +819,22 @@ class SqliteDocumentPersistence(object):
         self.cursor.execute("pragma optimize")
         self.connection.commit()
         self.cursor.execute("VACUUM")
-        self.connection.close()
-
-        self.connection = sqlite3.connect(self.current_filename)
         self.cursor = self.connection.cursor()
         self.cursor.execute("PRAGMA journal_mode=OFF")
         self.cursor.execute("pragma temp_store = memory")
         self.cursor.execute("pragma mmap_size = 30000000000")
 
+    def dump_in_memory_db_to_file(self):
+        # Connect to a new or existing database file
+        disk_conn = sqlite3.connect(self.current_filename)
+
+        # Use the backup API to copy the in-memory database to the disk file
+        with disk_conn:
+            self.connection.backup(disk_conn)
+
+        # Close the file-based database connection
+        disk_conn.close()
+
     def get_bytes(self):
         """
         Retrieves the document as bytes.
@@ -803,6 +843,10 @@ class SqliteDocumentPersistence(object):
             bytes: The document as bytes.
         """
         self.sync()
+
+        if self.inmemory:
+            self.dump_in_memory_db_to_file()
+
         with open(self.current_filename, "rb") as f:
             return f.read()
 
@@ -820,8 +864,8 @@ class SqliteDocumentPersistence(object):
 
         features = []
         for feature in self.cursor.execute(
-            "select id, cn_id, f_type, binary_value, single from ft where cn_id = ?",
-            [node.uuid],
+                "select id, cn_id, f_type, binary_value, single from ft where cn_id = ?",
+                [node.uuid],
         ).fetchall():
             feature_type_name = self.feature_type_names[feature[2]]
             single = feature[4] == 1
@@ -870,26 +914,27 @@ class SqliteDocumentPersistence(object):
         def get_all_node_ids(node):
             """
             This function recursively traverses a node tree, collecting the ids of all non-virtual nodes.
-
-            Args:
-                node (Node): The root node to start the traversal from.
-
-            Returns:
-                list: A list of ids of all non-virtual nodes in the tree.
             """
             all_node_ids = []
             if not node.virtual:
-                all_node_ids.append([node.uuid])
+                all_node_ids.append(node.uuid)  # Append the uuid directly, not as a list
                 for child in node.get_children():
                     all_node_ids.extend(get_all_node_ids(child))
-
             return all_node_ids
 
         all_child_ids = get_all_node_ids(node)
+        parameter_tuples = [(id,) for id in all_child_ids]  # Prepare the parameters as tuples
 
-        self.cursor.executemany("delete from cnp where cn_id=?", all_child_ids)
-        self.cursor.executemany("delete from cn where id=?", all_child_ids)
-        self.cursor.executemany("delete from ft where cn_id=?", all_child_ids)
+        # Assuming `self.cursor` is part of a larger transaction management system
+        try:
+            self.cursor.executemany("delete from cnp where cn_id=?", parameter_tuples)
+            self.cursor.executemany("delete from cn where id=?", parameter_tuples)
+            self.cursor.executemany("delete from ft where cn_id=?", parameter_tuples)
+            self.connection.commit()  # Commit the transaction if part of one
+            return all_child_ids
+        except Exception as e:
+            self.connection.rollback()  # Rollback in case of error
+            logger.error(f"An error occurred: {e}")
 
     def remove_all_features(self, node):
         """
@@ -935,9 +980,9 @@ class SqliteDocumentPersistence(object):
         """
         content_nodes = []
         if tag_uuid is None:
-            query = f"select cn_id from ft where f_type in (select id from f_type where name like 'tag:{tag}')"
+            query = f"select distinct(cn_id) from ft where f_type in (select id from f_type where name like 'tag:{tag}')"
         else:
-            query = f"select cn_id from ft where f_type in (select id from f_type where name like 'tag:{tag}') and tag_uuid = '{tag_uuid}'"
+            query = f"select distinct(cn_id) from ft where f_type in (select id from f_type where name like 'tag:{tag}') and tag_uuid = '{tag_uuid}'"
         for content_node_ids in self.cursor.execute(query).fetchall():
             content_nodes.append(self.get_node(content_node_ids[0]))
 
@@ -1037,7 +1082,7 @@ class SqliteDocumentPersistence(object):
             list: A list of all nodes with tags in the document.
         """
         content_nodes = []
-        query = "select cn_id from ft where f_type in (select id from f_type where name like 'tag:%')"
+        query = "select distinct(cn_id) from ft where f_type in (select id from f_type where name like 'tag:%')"
         for content_node_ids in self.cursor.execute(query).fetchall():
             content_nodes.append(self.get_node(content_node_ids[0]))
 
@@ -1095,14 +1140,14 @@ class SimpleObjectCache(object):
         self.objs[obj.uuid] = obj
         self.dirty_objs.add(obj.uuid)
 
-    def remove_obj(self, obj):
+    def remove_obj(self, obj: ContentNode):
         """
         Remove an object from the cache.
 
         Args:
             obj (object): The object to remove.
         """
-        if obj.uuid in self.objs:
+        if obj and obj.uuid in self.objs:
             self.objs.pop(obj.uuid)
             if obj.uuid in self.dirty_objs:
                 self.dirty_objs.remove(obj.uuid)
@@ -1168,7 +1213,7 @@ class PersistenceManager(object):
     This is implemented to allow us to work with large complex documents in a performance centered way.
     """
 
-    def __init__(self, document: Document, filename: str = None, delete_on_close=False):
+    def __init__(self, document: Document, filename: str = None, delete_on_close=False, inmemory=False):
         self.document = document
         self.node_cache = SimpleObjectCache()
         self.child_cache = {}
@@ -1178,9 +1223,24 @@ class PersistenceManager(object):
         self.node_parent_cache = {}
 
         self._underlying_persistence = SqliteDocumentPersistence(
-            document, filename, delete_on_close
+            document, filename, delete_on_close, inmemory=inmemory
         )
 
+    def get_node_by_uuid(self, uuid: int) -> ContentNode:
+        """
+        Retrieves a node by its uuid.
+
+        Args:
+            uuid (str): The uuid of the node to be retrieved.
+
+        Returns:
+            ContentNode: The node with the given uuid.
+        """
+        if self.node_cache.get_obj(uuid) is None:
+            self.node_cache.add_obj(self._underlying_persistence.get_node(uuid))
+
+        return self.node_cache.get_obj(uuid) # return the cached version
+
     def add_model_insight(self, model_insight: ModelInsight):
         """
         Adds a model insight to the underlying persistence layer.
@@ -1427,8 +1487,8 @@ class PersistenceManager(object):
             update_child_cache = True
 
         if (
-            node.uuid in self.node_parent_cache
-            and node._parent_uuid != self.node_parent_cache[node.uuid]
+                node.uuid in self.node_parent_cache
+                and node._parent_uuid != self.node_parent_cache[node.uuid]
         ):
             # Remove from the old parent
             self.child_id_cache[self.node_parent_cache[node.uuid]].remove(node.uuid)
@@ -1446,8 +1506,8 @@ class PersistenceManager(object):
                     self.child_id_cache[node._parent_uuid].add(node.uuid)
                     current_children = self.child_cache[node._parent_uuid]
                     if (
-                        len(current_children) == 0
-                        or node.index >= current_children[-1].index
+                            len(current_children) == 0
+                            or node.index >= current_children[-1].index
                     ):
                         self.child_cache[node._parent_uuid].append(node)
                     else:
@@ -1509,7 +1569,14 @@ class PersistenceManager(object):
         self.content_parts_cache.pop(node.uuid, None)
         self.feature_cache.pop(node.uuid, None)
 
-        self._underlying_persistence.remove_content_node(node)
+        all_ids = self._underlying_persistence.remove_content_node(node)
+
+        # remove all the ids from the cache
+        for id in all_ids:
+            tmp_node = self.node_cache.get_obj(id)
+            if tmp_node is not None:
+                self.node_cache.remove_obj(tmp_node)
+            self.node_cache.dirty_objs.remove(id) if id in self.node_cache.dirty_objs else None
 
     def get_children(self, node):
         """
